/* $Id: suresign.c,v 1.4 2005/04/27 16:57:59 mike Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_suresign.h"

static function_entry suresign_functions[] = {
    PHP_FE(suresign_write_watermark, NULL)
    PHP_FE(suresign_read_watermark, NULL)
    PHP_FE(suresign_errstr, NULL)
    {NULL, NULL, NULL}
};

zend_module_entry suresign_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
    PHP_SURESIGN_EXTNAME,
    suresign_functions,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
#if ZEND_MODULE_API_NO >= 20010901
    PHP_SURESIGN_VERSION,
#endif
    STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_SURESIGN
ZEND_GET_MODULE(suresign)
#endif


#include <sslib.h>


/*
   proto long suresign_write_watermark(string id, long weight,
				       string infile, string outfile)

   Arguments:
   	id	ID to be written into the watermarked file
	weight	Weight of watermark to be written (see SureSign docs)
	infile	Name of source file to be read
	outfile	Name of destination file to be written

   Returns a integer -- 0 on success, otherwise an error-code that can
   be translated into a human-readable message using
   suresign_errstr().
*/
PHP_FUNCTION(suresign_write_watermark)
{
    char *id;
    int id_len;
    long weight;
    char *infile;
    int infile_len;
    char *outfile;
    int outfile_len;
    long errnum;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "slss",
			      &id, &id_len,
			      &weight,
			      &infile, &infile_len,
			      &outfile, &outfile_len
			      ) == FAILURE)
        return;

    errnum = suresign_write_watermark(id, weight, infile, outfile);
    RETURN_LONG(errnum);
}


/*
   proto array suresign_read_watermark(string infile)

   Arguments:
	infile	Name of image file to be read

   Returns a five-element array as follows:
	0	error-code (0 for success)
	1	detected ID
	2	confidence with which the ID was detected
	3	creator
	4	confidence with which the creator was detected
   If the error-code is non-zero, the other elements are not filled
   in.  The error-code can be translated into a human-readable message
   using suresign_errstr().
*/
PHP_FUNCTION(suresign_read_watermark)
{
    char *infile;
    int infile_len;
    long errnum;
    char id[40];	   /* sizeof(SureSign_BasicBlock.lpImageID) */
    long idconfidence;
    char creator[40];	   /* sizeof(SureSign_BasicBlock.lpCreatorID) */
    long creatorconfidence;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
			      &infile, &infile_len) == FAILURE)
        return;

    errnum = suresign_read_watermark(infile,
				     id, &idconfidence,
				     creator, &creatorconfidence);
    array_init(return_value);
    add_index_long(return_value, 0, errnum);
    if (errnum == 0) {
	add_index_string(return_value, 1, id, 1);
	add_index_long(return_value, 2, idconfidence);
	add_index_string(return_value, 3, creator, 1);
	add_index_long(return_value, 4, creatorconfidence);
    }
}


/*
   proto string suresign_errstr(long errnum)

   Arguments:
	errnum	A SureSign error number

   Returns a string containing a human-readable error message
   corresponding to the error-code.
*/
PHP_FUNCTION(suresign_errstr)
{
    long errnum;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &errnum)
	== FAILURE)
        return;

    RETURN_STRING(suresign_errstr(errnum), 1);
}
