/* $Id: sslib.c,v 1.6 2005/04/25 11:14:59 mike Exp $ */

/*
 * This API fixes a few of the idiocies of the "suresign_simple"
 * interface.  For example, the error-number zero is always used to
 * indicate success, never Signum_ProcessComplete.
 */

#include <stdio.h>
#include <string.h>
#include "sslib.h"
#include "suresign_basic.h"

extern "C"
long suresign_write_watermark(char *id, long weight,
			      char *infile, char *outfile) {
    SureSign_BasicBlock block;
    long errnum;

    memset((void*) &block, 0, sizeof block);
    block.lpFileName = infile;
    if (strlen(id) >= 40)
	return Signum_ParamError; /* overloaded to mean "ID too long" */
    strcpy(block.lpImageID, id);
    block.lpOutputFile = outfile;

    errnum = SureSign_WatermarkFile(&block);
    return errnum == Signum_ProcessComplete ? 0 : errnum;
}


extern "C"
long suresign_read_watermark(char *infile,
			     char *idbuf, long *idconfidencep,
			     char *creatorbuf, long *creatorconfidencep) {
    SureSign_BasicBlock block;
    long errnum;

    memset((void*) &block, 0, sizeof block);
    block.lpFileName = infile;

    errnum = SureSign_DetectFile(&block);
    if (errnum != 0 && errnum != Signum_ProcessComplete)
	return errnum;

    strcpy(idbuf, block.lpImageID);
    *idconfidencep = block.nImageConf;
    strcpy(creatorbuf, block.lpCreatorID);
    *creatorconfidencep = block.nCreatorConf;

    return 0;
}


/* ### This really, really ought to be provided by the library */
extern "C"
char *suresign_errstr(long errnum) {
    /* long enough to print -2^63 into */
    static char buf[36];

    switch (errnum) {
    case Signum_No_error:
	return "No error";
    case Signum_Logo_Too_Large:
	return "The Logo is too large to be added to the image";
    case Signum_Logo_Wrong_Type:
	return "The Logo has the wrong number of separations";
    case Signum_NoLogoBufferSupplied:
	return "A logo has been requested but the logo buffer is empty";
    case Signum_LogoHardnessOutOfRange:
	return "If you want a logo to be written, the logo hardness must be in the range of 0 - 255.";
    case Signum_ProcessComplete:
	return "Processing is complete";
    case Signum_Err_memory_DeAllocator:
	return "The library requires a procedure to deallocate memory.";
    case Signum_Unsupported_PixelSample:
	return "Number of samples per pixel are 1,3, or 4.";
    case Signum_PrivateStorageNotInitialised:
	return "You called Signum_Print_Continue when Signum_Print_Init failed.";
    case Signum_PrivateStorageCorrupt:
	return "Private storage was overwritten.";
    case Signum_WatermarkAlreadyExists:
	return "The image already contains an HWS watermark.";
    case Signum_Insufficient_Mem_ForPrivateStore:
	return "There is not enough memory for the private storage.";
    case Signum_Action_Error:
	return "The action was not recognised.";
    case Signum_UnSupported_Data_Error:
	return "The data type is not supported (for example 4 bits per pixel)";
    case Signum_InternalError1:
	return "Internal error diagnostics. Should never occur.";
    case Signum_InternalError2:
	return "Internal error diagnostics. Should never occur.";
    case Signum_InternalError3:
	return "Internal error diagnostics. Should never occur.";
    case Signum_ParamError:
	return "A parameter is incorrect.";
    case Signum_FeatureNotAvailable:
	return "This Feature has been stubbed out of this version of the lib.";
    case Signum_InvalidIDs:
	return "The IDs strings are not of the correct form.";
    case Signum_Err_General:
	return "Internal error diagnostics. Should never occur.";
    case Signum_Err_PhaseError:
	return "Internal error diagnostics. Should never occur.";
    case Signum_NoWatermarkDetected:
	return "No watermark was detected.";
    case Signum_SecurityFailed:
	return "The access code is incorrect.";
    case Signum_Access_Code_Out_Of_Date:
	return "The access code is out of date";
    case Signum_Invalid_Format:
	return "format is too long or too short";
    case Signum_Cannot_Convert_Format:
	return "format resolves to too many or too few bits";
    case Signum_Internal_Number_Overflow:
	return "internal number representation too large";
    case Signum_Invalid_Blocks:
	return "block dimensions are invalid";
    case Signum_Watermark_Part_Detected:
	return "Only the User ID was found - the Image ID was not completely decoded";
    case Signum_Watermark_Badly_Formatted:
	return "A watermark has been found but is incorrectly formatted";
    case Signum_Image_Too_Small:
	return "The JPEG image is too small or does not contain enough information to be processed";
    case Signum_HashOutOfRange:
	return "The hash value is too large";
    case Signum_HashSizeLargerThanEncryptedSize:
	return "The number of hash bits exceeds the number of encrypted bits";
    case Signum_NotEnoughSamplesToEmbed:
	return "The block size is too small to embed all the required data";
    case Signum_JPEG_Corrupted:
	return "The JPEG file has been corrupted since validation";
    case Signum_Reading_Header_Error:
	return "Unable to read the JPEG header. This is returned if a progressive JPEG is processed";
    case Signum_Huff_Reading_Error:
	return "There is a problem reading the Huffman code";
    case Signum_Huff_Encoding_Error:
	return "There is a problem encoding the Huffman code";
    case Signum_No_Mem_Alloc_Function:
	return "No memory allocation callback was supplied";
    case Signum_No_Mem_Free_Function:
	return "No memory freeing callback was supplied";
    case Signum_No_Assert_Proc:
	return "No assert callback (for debugging) was supplied";
    case Signum_Hash_Block_Size_Out_Of_Range:
	return "The hash block size is too small / large";
    case Signum_No_Of_Hash_Bits_Out_Of_Range:
	return "The number of hash bits is too small / large";
    case Signum_Regions_Not_Defined:
	return "The region info part of the parameter block is not correct";
    case Signum_Buffer_NULL:
	return "The image buffer (input or output) is NULL";
    case Signum_Error_Unsupported_File_Type:
	return "The file type is not supported";
    case Signum_Error_Unable_To_Open_Input_File:
	return "The input file does not exist or is not available for reading";
    case Signum_Multiple_Page_Tiffs_Not_Supported:
	return "An attempt to process a Multi-page TIFF was made";
    case Signum_Out_Of_Memory:
	return "A memory allocation failed";
    case Signum_Cannot_Open_Output_File:
	return "The output file was not opened";
    case Signum_Invalid_Heaviness_Setting:
	return "An incorrect watermark heaviness was requested";
    case Signum_File_Read_Error_In_Libtiff:
	return "Libtiff produced a file read error";
    case Signum_Write_Read_Error_In_Libtiff:
	return "Libtiff produced a file write error";
    case Signum_File_Not_Found:
	return "The file does not exist";
    case Signum_Folder_Creation_Error:
	return "An attempt to create a folder failed";
    case Signum_Cannot_Copy:
	return "An attempt to copy data failed";
    default:
	break;
    }

    sprintf(buf, "Unknown error %ld", errnum);
    return buf;
}
